# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import bess_msg_pb2 as bess__msg__pb2


class BESSControlStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.GetVersion = channel.unary_unary(
        '/bess.pb.BESSControl/GetVersion',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.VersionResponse.FromString,
        )
    self.ResetAll = channel.unary_unary(
        '/bess.pb.BESSControl/ResetAll',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.KillBess = channel.unary_unary(
        '/bess.pb.BESSControl/KillBess',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ImportPlugin = channel.unary_unary(
        '/bess.pb.BESSControl/ImportPlugin',
        request_serializer=bess__msg__pb2.ImportPluginRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.UnloadPlugin = channel.unary_unary(
        '/bess.pb.BESSControl/UnloadPlugin',
        request_serializer=bess__msg__pb2.UnloadPluginRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ListPlugins = channel.unary_unary(
        '/bess.pb.BESSControl/ListPlugins',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListPluginsResponse.FromString,
        )
    self.PauseAll = channel.unary_unary(
        '/bess.pb.BESSControl/PauseAll',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.PauseWorker = channel.unary_unary(
        '/bess.pb.BESSControl/PauseWorker',
        request_serializer=bess__msg__pb2.PauseWorkerRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ResumeWorker = channel.unary_unary(
        '/bess.pb.BESSControl/ResumeWorker',
        request_serializer=bess__msg__pb2.ResumeWorkerRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ResumeAll = channel.unary_unary(
        '/bess.pb.BESSControl/ResumeAll',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ResetWorkers = channel.unary_unary(
        '/bess.pb.BESSControl/ResetWorkers',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ListWorkers = channel.unary_unary(
        '/bess.pb.BESSControl/ListWorkers',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListWorkersResponse.FromString,
        )
    self.AddWorker = channel.unary_unary(
        '/bess.pb.BESSControl/AddWorker',
        request_serializer=bess__msg__pb2.AddWorkerRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.DestroyWorker = channel.unary_unary(
        '/bess.pb.BESSControl/DestroyWorker',
        request_serializer=bess__msg__pb2.DestroyWorkerRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ResetTcs = channel.unary_unary(
        '/bess.pb.BESSControl/ResetTcs',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ListTcs = channel.unary_unary(
        '/bess.pb.BESSControl/ListTcs',
        request_serializer=bess__msg__pb2.ListTcsRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListTcsResponse.FromString,
        )
    self.CheckSchedulingConstraints = channel.unary_unary(
        '/bess.pb.BESSControl/CheckSchedulingConstraints',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CheckSchedulingConstraintsResponse.FromString,
        )
    self.AddTc = channel.unary_unary(
        '/bess.pb.BESSControl/AddTc',
        request_serializer=bess__msg__pb2.AddTcRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.UpdateTcParams = channel.unary_unary(
        '/bess.pb.BESSControl/UpdateTcParams',
        request_serializer=bess__msg__pb2.UpdateTcParamsRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.UpdateTcParent = channel.unary_unary(
        '/bess.pb.BESSControl/UpdateTcParent',
        request_serializer=bess__msg__pb2.UpdateTcParentRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.GetTcStats = channel.unary_unary(
        '/bess.pb.BESSControl/GetTcStats',
        request_serializer=bess__msg__pb2.GetTcStatsRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetTcStatsResponse.FromString,
        )
    self.ListDrivers = channel.unary_unary(
        '/bess.pb.BESSControl/ListDrivers',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListDriversResponse.FromString,
        )
    self.GetDriverInfo = channel.unary_unary(
        '/bess.pb.BESSControl/GetDriverInfo',
        request_serializer=bess__msg__pb2.GetDriverInfoRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetDriverInfoResponse.FromString,
        )
    self.ResetPorts = channel.unary_unary(
        '/bess.pb.BESSControl/ResetPorts',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ListPorts = channel.unary_unary(
        '/bess.pb.BESSControl/ListPorts',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListPortsResponse.FromString,
        )
    self.CreatePort = channel.unary_unary(
        '/bess.pb.BESSControl/CreatePort',
        request_serializer=bess__msg__pb2.CreatePortRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CreatePortResponse.FromString,
        )
    self.DestroyPort = channel.unary_unary(
        '/bess.pb.BESSControl/DestroyPort',
        request_serializer=bess__msg__pb2.DestroyPortRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.SetPortConf = channel.unary_unary(
        '/bess.pb.BESSControl/SetPortConf',
        request_serializer=bess__msg__pb2.SetPortConfRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CommandResponse.FromString,
        )
    self.GetPortConf = channel.unary_unary(
        '/bess.pb.BESSControl/GetPortConf',
        request_serializer=bess__msg__pb2.GetPortConfRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetPortConfResponse.FromString,
        )
    self.GetPortStats = channel.unary_unary(
        '/bess.pb.BESSControl/GetPortStats',
        request_serializer=bess__msg__pb2.GetPortStatsRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetPortStatsResponse.FromString,
        )
    self.GetLinkStatus = channel.unary_unary(
        '/bess.pb.BESSControl/GetLinkStatus',
        request_serializer=bess__msg__pb2.GetLinkStatusRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetLinkStatusResponse.FromString,
        )
    self.ListMclass = channel.unary_unary(
        '/bess.pb.BESSControl/ListMclass',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListMclassResponse.FromString,
        )
    self.GetMclassInfo = channel.unary_unary(
        '/bess.pb.BESSControl/GetMclassInfo',
        request_serializer=bess__msg__pb2.GetMclassInfoRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetMclassInfoResponse.FromString,
        )
    self.ResetModules = channel.unary_unary(
        '/bess.pb.BESSControl/ResetModules',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.ListModules = channel.unary_unary(
        '/bess.pb.BESSControl/ListModules',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListModulesResponse.FromString,
        )
    self.CreateModule = channel.unary_unary(
        '/bess.pb.BESSControl/CreateModule',
        request_serializer=bess__msg__pb2.CreateModuleRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CreateModuleResponse.FromString,
        )
    self.DestroyModule = channel.unary_unary(
        '/bess.pb.BESSControl/DestroyModule',
        request_serializer=bess__msg__pb2.DestroyModuleRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.GetModuleInfo = channel.unary_unary(
        '/bess.pb.BESSControl/GetModuleInfo',
        request_serializer=bess__msg__pb2.GetModuleInfoRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetModuleInfoResponse.FromString,
        )
    self.ConnectModules = channel.unary_unary(
        '/bess.pb.BESSControl/ConnectModules',
        request_serializer=bess__msg__pb2.ConnectModulesRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.DisconnectModules = channel.unary_unary(
        '/bess.pb.BESSControl/DisconnectModules',
        request_serializer=bess__msg__pb2.DisconnectModulesRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.EmptyResponse.FromString,
        )
    self.DumpMempool = channel.unary_unary(
        '/bess.pb.BESSControl/DumpMempool',
        request_serializer=bess__msg__pb2.DumpMempoolRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.DumpMempoolResponse.FromString,
        )
    self.ModuleCommand = channel.unary_unary(
        '/bess.pb.BESSControl/ModuleCommand',
        request_serializer=bess__msg__pb2.CommandRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CommandResponse.FromString,
        )
    self.ListGateHookClass = channel.unary_unary(
        '/bess.pb.BESSControl/ListGateHookClass',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListGateHookClassResponse.FromString,
        )
    self.GetGateHookClassInfo = channel.unary_unary(
        '/bess.pb.BESSControl/GetGateHookClassInfo',
        request_serializer=bess__msg__pb2.GetGateHookClassInfoRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.GetGateHookClassInfoResponse.FromString,
        )
    self.ConfigureGateHook = channel.unary_unary(
        '/bess.pb.BESSControl/ConfigureGateHook',
        request_serializer=bess__msg__pb2.ConfigureGateHookRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ConfigureGateHookResponse.FromString,
        )
    self.ListGateHooks = channel.unary_unary(
        '/bess.pb.BESSControl/ListGateHooks',
        request_serializer=bess__msg__pb2.EmptyRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.ListGateHooksResponse.FromString,
        )
    self.GateHookCommand = channel.unary_unary(
        '/bess.pb.BESSControl/GateHookCommand',
        request_serializer=bess__msg__pb2.GateHookCommandRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CommandResponse.FromString,
        )
    self.ConfigureResumeHook = channel.unary_unary(
        '/bess.pb.BESSControl/ConfigureResumeHook',
        request_serializer=bess__msg__pb2.ConfigureResumeHookRequest.SerializeToString,
        response_deserializer=bess__msg__pb2.CommandResponse.FromString,
        )


class BESSControlServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def GetVersion(self, request, context):
    """-------------------------------------------------------------------------
    System
    -------------------------------------------------------------------------

    / Query version of bessd
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResetAll(self, request, context):
    """/ Reset the current packet processing datapath to the initial state.
    /
    / This command is identical to the following sequence:
    /   ResetModules()
    /   ResetPorts()
    /   ResetTcs()
    /   ResetWorkers()
    / As it clears everything, BESS should appear as if the daemon has freshly
    / started (if not, it is a bug; please report).
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def KillBess(self, request, context):
    """/ Terminate the BESS daemon.
    /
    / BESS daemon shuts off in a graceful manner. Note that this command is
    / "asynchronous": this command doesn't block until the BESS daemon has
    / shut off.
    /
    / NOTE: There should be no running worker to run this command.
    / FIXME: rename (e.g., Terminate)
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ImportPlugin(self, request, context):
    """/ Import a plugin
    /
    / At the moment plugins can only contain module types,
    / but might also support drivers/hooks/schedulers in the future.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UnloadPlugin(self, request, context):
    """/ Unload a plugin
    /
    / At the moment plugins can only contain module types,
    / but might also support drivers/hooks/schedulers in the future.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListPlugins(self, request, context):
    """/ List imported plugins
    /
    / At the moment plugins can only contain module types,
    / but might also support drivers/hooks/schedulers in the future.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PauseAll(self, request, context):
    """-------------------------------------------------------------------------
    Worker
    -------------------------------------------------------------------------

    / Pause all running workers temporarily
    /
    / Some RPC commands to BESS or individual modules/ports require that
    / threads must be inactive, to avoid race conditions.
    / For such commands, use PauseALl at the beginning and ResumeAll at the end.
    /  PauseAll()
    /   SomeCommand1()
    /   SomeCommand2()
    /   ...
    /  ResumeAll()
    / Keep the duration as short as possible, to avoid packet drops.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PauseWorker(self, request, context):
    """/ Pause the specified worker temporarily
    /
    / Some RPC commands to BESS or individual modules/ports require that
    / threads must be inactive, to avoid race conditions.
    / For such commands, use PauseWorker at the beginning and ResumeWorker at the end.
    /  PauseWorker(0)
    /   SomeCommand1()
    /   SomeCommand2()
    /   ...
    /  ResumeWorker(0)
    / Keep the duration as short as possible, to avoid packet drops.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResumeWorker(self, request, context):
    """/ Resume the specified worker
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResumeAll(self, request, context):
    """/ Resume all paused workers
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResetWorkers(self, request, context):
    """/ Remove all existing workers
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListWorkers(self, request, context):
    """/ Enumerate all existing workers
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def AddWorker(self, request, context):
    """/ Create a new worker
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DestroyWorker(self, request, context):
    """/ Remove a single worker
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResetTcs(self, request, context):
    """-------------------------------------------------------------------------
    Traffic classe & task
    -------------------------------------------------------------------------

    / Remove all existing traffic classes
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListTcs(self, request, context):
    """/ Enumerate all existing workers
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CheckSchedulingConstraints(self, request, context):
    """/ Check scheduling contraints
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def AddTc(self, request, context):
    """/ Create a new traffic class
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateTcParams(self, request, context):
    """/ Update parameters of an existing traffic class
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def UpdateTcParent(self, request, context):
    """/ Change parent (and child arguments) of an existing traffic class
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetTcStats(self, request, context):
    """/ Collect statistics of a traffic class
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListDrivers(self, request, context):
    """-------------------------------------------------------------------------
    Port
    -------------------------------------------------------------------------

    / Enumerate all port drivers available
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetDriverInfo(self, request, context):
    """/ Query detailed information of a port driver
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResetPorts(self, request, context):
    """/ Remove all initialized ports
    /
    / Will fail if there are modules that are still using ports.
    / (e.g., PortInc, PortOut, QueueInc, QueueOut)
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListPorts(self, request, context):
    """/ Enumerate all initialized ports
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreatePort(self, request, context):
    """/ Create a new port from the specified driver
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DestroyPort(self, request, context):
    """/ Remove a port
    /
    / The port should not be being used by a port-related module.
    / (e.g., PortInc, PortOut, QueueInc, QueueOut)
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def SetPortConf(self, request, context):
    """/ Runtime-updatable configuration
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetPortConf(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetPortStats(self, request, context):
    """/ Collect port statistics
    /
    / At the moment, per-queue stats are not supported.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetLinkStatus(self, request, context):
    """/ Query link status
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListMclass(self, request, context):
    """TODO: Add PortCommand, like ModuleCommand, which performs driver-specific
    actions on a port.

    -------------------------------------------------------------------------
    Module
    -------------------------------------------------------------------------

    / Enumerate all module types available
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetMclassInfo(self, request, context):
    """/ Query detailed information of a module type
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ResetModules(self, request, context):
    """/ Remove all modules.
    /
    / This RPC will always succeed (unless there is a running worker)
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListModules(self, request, context):
    """/ Enumerate all initialized modules
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def CreateModule(self, request, context):
    """/ Create a new module instance from the given module type
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DestroyModule(self, request, context):
    """/ Destroy an exsting module
    /
    / If the module is connected to other modules' input/output gate, they are
    / disconnected first. All tasks created by the module will also be destoyed.
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetModuleInfo(self, request, context):
    """/ Fetch detailed information of an module instance
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ConnectModules(self, request, context):
    """/ Connect two modules.
    /
    / Connect between m1's ogate and n2's igate (i.e., ackets sent to m1's ogate
    / will be fed to m2's igate). The oate can be connected to only one igate,
    / while the igate can be connected to multiple output gates.
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DisconnectModules(self, request, context):
    """/ Disconnect two modules.
    /
    / It removes a connection between two modules (you specify the previous one
    / and its output gate). All packets coming out from the ogate will be
    / dropped. Once disconnected, the ogate can be connected
    / to any input gate.
    /
    / NOTE: There should be no running worker to run this command.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DumpMempool(self, request, context):
    """/ Dump various stats about BESS's packet pools
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ModuleCommand(self, request, context):
    """/ Send a command to the specified module instance.
    /
    / Each module type defines a list of modyle-specific commands, which
    / allow external programs to communicate with the module at runtime.
    / See module_msg.proto for details.
    /
    / NOTE: Some commands cannot be used if there are running workers.
    /       For those commands you must pause all workers first.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListGateHookClass(self, request, context):
    """-------------------------------------------------------------------------
    Gate hooks
    -------------------------------------------------------------------------

    / Enumerate all gatehook types available
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetGateHookClassInfo(self, request, context):
    """/ Query detailed information of a gatehook type
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ConfigureGateHook(self, request, context):
    """/ Enable/Disable a gate hook.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListGateHooks(self, request, context):
    """/ Enumerate all gatehook installed
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GateHookCommand(self, request, context):
    """/ Send command to gate hook instance.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ConfigureResumeHook(self, request, context):
    """-------------------------------------------------------------------------
    Resume hooks
    -------------------------------------------------------------------------

    / Enable/Disable a resume hook.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_BESSControlServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'GetVersion': grpc.unary_unary_rpc_method_handler(
          servicer.GetVersion,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.VersionResponse.SerializeToString,
      ),
      'ResetAll': grpc.unary_unary_rpc_method_handler(
          servicer.ResetAll,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'KillBess': grpc.unary_unary_rpc_method_handler(
          servicer.KillBess,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ImportPlugin': grpc.unary_unary_rpc_method_handler(
          servicer.ImportPlugin,
          request_deserializer=bess__msg__pb2.ImportPluginRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'UnloadPlugin': grpc.unary_unary_rpc_method_handler(
          servicer.UnloadPlugin,
          request_deserializer=bess__msg__pb2.UnloadPluginRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ListPlugins': grpc.unary_unary_rpc_method_handler(
          servicer.ListPlugins,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListPluginsResponse.SerializeToString,
      ),
      'PauseAll': grpc.unary_unary_rpc_method_handler(
          servicer.PauseAll,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'PauseWorker': grpc.unary_unary_rpc_method_handler(
          servicer.PauseWorker,
          request_deserializer=bess__msg__pb2.PauseWorkerRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ResumeWorker': grpc.unary_unary_rpc_method_handler(
          servicer.ResumeWorker,
          request_deserializer=bess__msg__pb2.ResumeWorkerRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ResumeAll': grpc.unary_unary_rpc_method_handler(
          servicer.ResumeAll,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ResetWorkers': grpc.unary_unary_rpc_method_handler(
          servicer.ResetWorkers,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ListWorkers': grpc.unary_unary_rpc_method_handler(
          servicer.ListWorkers,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListWorkersResponse.SerializeToString,
      ),
      'AddWorker': grpc.unary_unary_rpc_method_handler(
          servicer.AddWorker,
          request_deserializer=bess__msg__pb2.AddWorkerRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'DestroyWorker': grpc.unary_unary_rpc_method_handler(
          servicer.DestroyWorker,
          request_deserializer=bess__msg__pb2.DestroyWorkerRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ResetTcs': grpc.unary_unary_rpc_method_handler(
          servicer.ResetTcs,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ListTcs': grpc.unary_unary_rpc_method_handler(
          servicer.ListTcs,
          request_deserializer=bess__msg__pb2.ListTcsRequest.FromString,
          response_serializer=bess__msg__pb2.ListTcsResponse.SerializeToString,
      ),
      'CheckSchedulingConstraints': grpc.unary_unary_rpc_method_handler(
          servicer.CheckSchedulingConstraints,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.CheckSchedulingConstraintsResponse.SerializeToString,
      ),
      'AddTc': grpc.unary_unary_rpc_method_handler(
          servicer.AddTc,
          request_deserializer=bess__msg__pb2.AddTcRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'UpdateTcParams': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateTcParams,
          request_deserializer=bess__msg__pb2.UpdateTcParamsRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'UpdateTcParent': grpc.unary_unary_rpc_method_handler(
          servicer.UpdateTcParent,
          request_deserializer=bess__msg__pb2.UpdateTcParentRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'GetTcStats': grpc.unary_unary_rpc_method_handler(
          servicer.GetTcStats,
          request_deserializer=bess__msg__pb2.GetTcStatsRequest.FromString,
          response_serializer=bess__msg__pb2.GetTcStatsResponse.SerializeToString,
      ),
      'ListDrivers': grpc.unary_unary_rpc_method_handler(
          servicer.ListDrivers,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListDriversResponse.SerializeToString,
      ),
      'GetDriverInfo': grpc.unary_unary_rpc_method_handler(
          servicer.GetDriverInfo,
          request_deserializer=bess__msg__pb2.GetDriverInfoRequest.FromString,
          response_serializer=bess__msg__pb2.GetDriverInfoResponse.SerializeToString,
      ),
      'ResetPorts': grpc.unary_unary_rpc_method_handler(
          servicer.ResetPorts,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ListPorts': grpc.unary_unary_rpc_method_handler(
          servicer.ListPorts,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListPortsResponse.SerializeToString,
      ),
      'CreatePort': grpc.unary_unary_rpc_method_handler(
          servicer.CreatePort,
          request_deserializer=bess__msg__pb2.CreatePortRequest.FromString,
          response_serializer=bess__msg__pb2.CreatePortResponse.SerializeToString,
      ),
      'DestroyPort': grpc.unary_unary_rpc_method_handler(
          servicer.DestroyPort,
          request_deserializer=bess__msg__pb2.DestroyPortRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'SetPortConf': grpc.unary_unary_rpc_method_handler(
          servicer.SetPortConf,
          request_deserializer=bess__msg__pb2.SetPortConfRequest.FromString,
          response_serializer=bess__msg__pb2.CommandResponse.SerializeToString,
      ),
      'GetPortConf': grpc.unary_unary_rpc_method_handler(
          servicer.GetPortConf,
          request_deserializer=bess__msg__pb2.GetPortConfRequest.FromString,
          response_serializer=bess__msg__pb2.GetPortConfResponse.SerializeToString,
      ),
      'GetPortStats': grpc.unary_unary_rpc_method_handler(
          servicer.GetPortStats,
          request_deserializer=bess__msg__pb2.GetPortStatsRequest.FromString,
          response_serializer=bess__msg__pb2.GetPortStatsResponse.SerializeToString,
      ),
      'GetLinkStatus': grpc.unary_unary_rpc_method_handler(
          servicer.GetLinkStatus,
          request_deserializer=bess__msg__pb2.GetLinkStatusRequest.FromString,
          response_serializer=bess__msg__pb2.GetLinkStatusResponse.SerializeToString,
      ),
      'ListMclass': grpc.unary_unary_rpc_method_handler(
          servicer.ListMclass,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListMclassResponse.SerializeToString,
      ),
      'GetMclassInfo': grpc.unary_unary_rpc_method_handler(
          servicer.GetMclassInfo,
          request_deserializer=bess__msg__pb2.GetMclassInfoRequest.FromString,
          response_serializer=bess__msg__pb2.GetMclassInfoResponse.SerializeToString,
      ),
      'ResetModules': grpc.unary_unary_rpc_method_handler(
          servicer.ResetModules,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'ListModules': grpc.unary_unary_rpc_method_handler(
          servicer.ListModules,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListModulesResponse.SerializeToString,
      ),
      'CreateModule': grpc.unary_unary_rpc_method_handler(
          servicer.CreateModule,
          request_deserializer=bess__msg__pb2.CreateModuleRequest.FromString,
          response_serializer=bess__msg__pb2.CreateModuleResponse.SerializeToString,
      ),
      'DestroyModule': grpc.unary_unary_rpc_method_handler(
          servicer.DestroyModule,
          request_deserializer=bess__msg__pb2.DestroyModuleRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'GetModuleInfo': grpc.unary_unary_rpc_method_handler(
          servicer.GetModuleInfo,
          request_deserializer=bess__msg__pb2.GetModuleInfoRequest.FromString,
          response_serializer=bess__msg__pb2.GetModuleInfoResponse.SerializeToString,
      ),
      'ConnectModules': grpc.unary_unary_rpc_method_handler(
          servicer.ConnectModules,
          request_deserializer=bess__msg__pb2.ConnectModulesRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'DisconnectModules': grpc.unary_unary_rpc_method_handler(
          servicer.DisconnectModules,
          request_deserializer=bess__msg__pb2.DisconnectModulesRequest.FromString,
          response_serializer=bess__msg__pb2.EmptyResponse.SerializeToString,
      ),
      'DumpMempool': grpc.unary_unary_rpc_method_handler(
          servicer.DumpMempool,
          request_deserializer=bess__msg__pb2.DumpMempoolRequest.FromString,
          response_serializer=bess__msg__pb2.DumpMempoolResponse.SerializeToString,
      ),
      'ModuleCommand': grpc.unary_unary_rpc_method_handler(
          servicer.ModuleCommand,
          request_deserializer=bess__msg__pb2.CommandRequest.FromString,
          response_serializer=bess__msg__pb2.CommandResponse.SerializeToString,
      ),
      'ListGateHookClass': grpc.unary_unary_rpc_method_handler(
          servicer.ListGateHookClass,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListGateHookClassResponse.SerializeToString,
      ),
      'GetGateHookClassInfo': grpc.unary_unary_rpc_method_handler(
          servicer.GetGateHookClassInfo,
          request_deserializer=bess__msg__pb2.GetGateHookClassInfoRequest.FromString,
          response_serializer=bess__msg__pb2.GetGateHookClassInfoResponse.SerializeToString,
      ),
      'ConfigureGateHook': grpc.unary_unary_rpc_method_handler(
          servicer.ConfigureGateHook,
          request_deserializer=bess__msg__pb2.ConfigureGateHookRequest.FromString,
          response_serializer=bess__msg__pb2.ConfigureGateHookResponse.SerializeToString,
      ),
      'ListGateHooks': grpc.unary_unary_rpc_method_handler(
          servicer.ListGateHooks,
          request_deserializer=bess__msg__pb2.EmptyRequest.FromString,
          response_serializer=bess__msg__pb2.ListGateHooksResponse.SerializeToString,
      ),
      'GateHookCommand': grpc.unary_unary_rpc_method_handler(
          servicer.GateHookCommand,
          request_deserializer=bess__msg__pb2.GateHookCommandRequest.FromString,
          response_serializer=bess__msg__pb2.CommandResponse.SerializeToString,
      ),
      'ConfigureResumeHook': grpc.unary_unary_rpc_method_handler(
          servicer.ConfigureResumeHook,
          request_deserializer=bess__msg__pb2.ConfigureResumeHookRequest.FromString,
          response_serializer=bess__msg__pb2.CommandResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'bess.pb.BESSControl', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Copyright (c) 2014-2016, The Regents of the University of California.
# Copyright (c) 2016-2017, Nefeli Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# * Neither the names of the copyright holders nor the names of their
# contributors may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import absolute_import
import unittest
import grpc
import time
from concurrent import futures

from . import bess
from .builtin_pb import bess_msg_pb2 as bess_msg
from .builtin_pb import service_pb2


class DummyServiceImpl(service_pb2.BESSControlServicer):

    def __init__(self):
        pass

    def KillBess(self, request, context):
        return bess_msg.EmptyResponse()

    def CreatePort(self, request, context):
        response = bess_msg.CreatePortResponse()
        response.name = request.name
        return response

    def ModuleCommand(self, request, context):
        response = bess_msg.CommandResponse()
        return response

    def ListModules(self, request, context):
        response = bess_msg.ListModulesResponse()
        return response


class TestBESS(unittest.TestCase):
    # Do not use BESS.DEF_PORT (== 10514), as it might be being used by
    # a real bessd process.
    PORT = 19876
    GRPC_URL = 'localhost:' + str(PORT)

    @classmethod
    def setUpClass(cls):
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=2))
        service_pb2.add_BESSControlServicer_to_server(
            DummyServiceImpl(),
            server)
        server.add_insecure_port('[::]:%d' % cls.PORT)
        server.start()
        cls.server = server

    @classmethod
    def tearDownClass(cls):
        future = cls.server.stop(0)
        future.wait()

    def test_connect(self):
        client = bess.BESS()
        client.connect(grpc_url=self.GRPC_URL)
        time.sleep(0.1)
        self.assertEqual(True, client.is_connected())

        client.disconnect()
        time.sleep(0.1)
        self.assertEqual(False, client.is_connected())

    def test_kill(self):
        client = bess.BESS()
        client.connect(grpc_url=self.GRPC_URL)

        response = client.kill(block=False)
        self.assertEqual(0, response.error.code)

    def test_list_modules(self):
        client = bess.BESS()
        client.connect(grpc_url=self.GRPC_URL)

        response = client.list_modules()
        self.assertEqual(0, response.error.code)

    def test_create_port(self):
        client = bess.BESS()
        client.connect(grpc_url=self.GRPC_URL)

        response = client.create_port('PCAPPort', 'p0', {'dev': 'rnd'})
        self.assertEqual(0, response.error.code)
        self.assertEqual('p0', response.name)

        response = client.create_port('PMDPort', 'p0', {
            'loopback': True,
            'port_id': 14325,
            'pci': 'akshdkashf'})
        self.assertEqual(0, response.error.code)
        self.assertEqual('p0', response.name)

        response = client.create_port('UnixSocketPort', 'p0',
                                      {'path': '/ajksd/dd'})
        self.assertEqual(0, response.error.code)
        self.assertEqual('p0', response.name)

        response = client.create_port('VPort', 'p0', {
            'ifname': 'veth0',
            'container_pid': 23124,
            'rxq_cpus': [1, 2, 3],
            'tx_tci': 123,
            'tx_outer_tci': 123,
            'loopback': False,
            'ip_addrs': ['1.2.3.4', '255.254.253.252']
        })
        self.assertEqual(0, response.error.code)
        self.assertEqual('p0', response.name)

    def test_run_module_command(self):
        client = bess.BESS()
        client.connect(grpc_url=self.GRPC_URL)

        response = client.run_module_command('m1',
                                             'add',
                                             'ExactMatchCommandAddArg',
                                             {'gate': 0,
                                                 'fields': [{'value_bin': b'\x11'}, {'value_bin': b'\x22'}]})
        self.assertEqual(0, response.error.code)



# Copyright (c) 2014-2016, The Regents of the University of California.
# Copyright (c) 2016-2017, Nefeli Networks, Inc.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
# list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.
#
# * Neither the names of the copyright holders nor the names of their
# contributors may be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Copied from https://github.com/benhodgson/protobuf-to-dict (cd98280)
# written by Ben Hodgson, with additional fixes

import sys

from google.protobuf.message import Message
from google.protobuf.descriptor import FieldDescriptor


__all__ = ["protobuf_to_dict", "TYPE_CALLABLE_MAP",
           "dict_to_protobuf", "REVERSE_TYPE_CALLABLE_MAP"]


EXTENSION_CONTAINER = '___X'


TYPE_CALLABLE_MAP = {
    FieldDescriptor.TYPE_DOUBLE: float,
    FieldDescriptor.TYPE_FLOAT: float,
    FieldDescriptor.TYPE_INT32: int,
    FieldDescriptor.TYPE_INT64: int,
    FieldDescriptor.TYPE_UINT32: int,
    FieldDescriptor.TYPE_UINT64: int,
    FieldDescriptor.TYPE_SINT32: int,
    FieldDescriptor.TYPE_SINT64: int,
    FieldDescriptor.TYPE_FIXED32: int,
    FieldDescriptor.TYPE_FIXED64: int,
    FieldDescriptor.TYPE_SFIXED32: int,
    FieldDescriptor.TYPE_SFIXED64: int,
    FieldDescriptor.TYPE_BOOL: bool,
    FieldDescriptor.TYPE_STRING: str,
    FieldDescriptor.TYPE_BYTES: bytes,
    FieldDescriptor.TYPE_ENUM: int,
}


def repeated_map(type_k_callable, type_v_callable):
    return lambda values: {type_k_callable(k): type_v_callable(v)
                           for k, v in values.items()}


def repeated(type_callable):
    return lambda value_list: [type_callable(value) for value in value_list]


def enum_label_name(field, value):
    return field.enum_type.values_by_number[int(value)].name


def protobuf_to_dict(pb, type_callable_map=TYPE_CALLABLE_MAP, use_enum_labels=False):
    result_dict = {}
    extensions = {}
    for field, value in pb.ListFields():
        if field.type == FieldDescriptor.TYPE_MESSAGE and \
                field.message_type.GetOptions().map_entry:
            field_k = field.message_type.fields_by_name['key']
            type_k_callable = _get_field_value_adaptor(pb,
                                                       field_k,
                                                       type_callable_map,
                                                       use_enum_labels)
            field_v = field.message_type.fields_by_name['value']
            type_v_callable = _get_field_value_adaptor(pb,
                                                       field_v,
                                                       type_callable_map,
                                                       use_enum_labels)
            type_callable = repeated_map(type_k_callable, type_v_callable)
        else:
            type_callable = _get_field_value_adaptor(
                pb, field, type_callable_map, use_enum_labels)
            if field.label == FieldDescriptor.LABEL_REPEATED:
                type_callable = repeated(type_callable)

        if field.is_extension:
            extensions[str(field.number)] = type_callable(value)
            continue

        result_dict[field.name] = type_callable(value)

    if extensions:
        result_dict[EXTENSION_CONTAINER] = extensions
    return result_dict


def _get_field_value_adaptor(pb, field, type_callable_map=TYPE_CALLABLE_MAP, use_enum_labels=False):
    if field.type == FieldDescriptor.TYPE_MESSAGE:
        # recursively encode protobuf sub-message
        return lambda pb: protobuf_to_dict(pb,
                                           type_callable_map=type_callable_map,
                                           use_enum_labels=use_enum_labels)

    if use_enum_labels and field.type == FieldDescriptor.TYPE_ENUM:
        return lambda value: enum_label_name(field, value)

    if field.type in type_callable_map:
        return type_callable_map[field.type]

    raise TypeError("Field %s.%s has unrecognised type id %d" % (
        pb.__class__.__name__, field.name, field.type))


REVERSE_TYPE_CALLABLE_MAP = {
    # empty
}


def dict_to_protobuf(pb_klass_or_instance, values, type_callable_map=REVERSE_TYPE_CALLABLE_MAP, strict=True):
    """Populates a protobuf model from a dictionary.

    :param pb_klass_or_instance: a protobuf message class, or an protobuf instance
    :type pb_klass_or_instance: a type or instance of a subclass of google.protobuf.message.Message
    :param dict values: a dictionary of values. Repeated and nested values are
       fully supported.
    :param dict type_callable_map: a mapping of protobuf types to callables for setting
       values on the target instance.
    :param bool strict: complain if keys in the map are not fields on the message.
    """
    if isinstance(pb_klass_or_instance, Message):
        instance = pb_klass_or_instance
    else:
        instance = pb_klass_or_instance()
    return _dict_to_protobuf(instance, values, type_callable_map, strict)


def _get_field_mapping(pb, dict_value, strict):
    field_mapping = []
    for key, value in dict_value.items():
        if key == EXTENSION_CONTAINER:
            continue
        if key not in pb.DESCRIPTOR.fields_by_name:
            if strict:
                raise KeyError("%s does not have a field called %s" %
                               (pb.__class__.__name__, key))
            continue
        field_mapping.append((pb.DESCRIPTOR.fields_by_name[
                             key], value, getattr(pb, key, None)))

    for ext_num, ext_val in dict_value.get(EXTENSION_CONTAINER, {}).items():
        try:
            ext_num = int(ext_num)
        except ValueError:
            raise ValueError("Extension keys must be integers.")
        if ext_num not in pb._extensions_by_number:
            if strict:
                raise KeyError("%s does not have a extension with number %s. "
                               "Perhaps you forgot to import it?" %
                               (pb.__class__.__name__, key))
            continue
        ext_field = pb._extensions_by_number[ext_num]
        pb_val = pb.Extensions[ext_field]
        field_mapping.append((ext_field, ext_val, pb_val))

    return field_mapping


def _dict_to_protobuf(pb, value, type_callable_map, strict):
    fields = _get_field_mapping(pb, value, strict)

    if sys.version_info[0] == 2:
        basestr = basestring
    else:
        basestr = str

    for field, input_value, pb_value in fields:
        if field.label == FieldDescriptor.LABEL_REPEATED:
            if field.message_type and field.message_type.has_options and \
                   field.message_type.GetOptions().map_entry:
                # Special processing for nested dict
                if isinstance(input_value, dict) and all([isinstance(x, dict) for x in input_value.values()]):
                    for k, v in input_value.items():
                        _dict_to_protobuf(
                            pb_value[k], input_value[k], type_callable_map, strict)
                else:
                    pb_value.update(input_value)
                continue
            for item in input_value:
                if field.type == FieldDescriptor.TYPE_MESSAGE:
                    m = pb_value.add()
                    _dict_to_protobuf(m, item, type_callable_map, strict)
                elif field.type == FieldDescriptor.TYPE_ENUM and isinstance(item, basestr):
                    pb_value.append(_string_to_enum(field, item))
                else:
                    pb_value.append(item)
            continue
        if field.type == FieldDescriptor.TYPE_MESSAGE:
            _dict_to_protobuf(pb_value, input_value, type_callable_map, strict)
            continue

        if field.type in type_callable_map:
            input_value = type_callable_map[field.type](input_value)

        if field.is_extension:
            pb.Extensions[field] = input_value
            continue

        if field.type == FieldDescriptor.TYPE_ENUM and isinstance(input_value, basestr):
            input_value = _string_to_enum(field, input_value)

        setattr(pb, field.name, input_value)

    return pb


def _string_to_enum(field, input_value):
    enum_dict = field.enum_type.values_by_name
    try:
        input_value = enum_dict[input_value].number
    except KeyError:
        raise KeyError("`%s` is not a valid value for field `%s`" %
                       (input_value, field.name))
    return input_value



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: service.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import bess_msg_pb2 as bess__msg__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='service.proto',
  package='bess.pb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=b'\n\rservice.proto\x12\x07\x62\x65ss.pb\x1a\x0e\x62\x65ss_msg.proto2\xd4\x1b\n\x0b\x42\x45SSControl\x12?\n\nGetVersion\x12\x15.bess.pb.EmptyRequest\x1a\x18.bess.pb.VersionResponse\"\x00\x12;\n\x08ResetAll\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12;\n\x08KillBess\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x46\n\x0cImportPlugin\x12\x1c.bess.pb.ImportPluginRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x46\n\x0cUnloadPlugin\x12\x1c.bess.pb.UnloadPluginRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x44\n\x0bListPlugins\x12\x15.bess.pb.EmptyRequest\x1a\x1c.bess.pb.ListPluginsResponse\"\x00\x12;\n\x08PauseAll\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x44\n\x0bPauseWorker\x12\x1b.bess.pb.PauseWorkerRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x46\n\x0cResumeWorker\x12\x1c.bess.pb.ResumeWorkerRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12<\n\tResumeAll\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12?\n\x0cResetWorkers\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x44\n\x0bListWorkers\x12\x15.bess.pb.EmptyRequest\x1a\x1c.bess.pb.ListWorkersResponse\"\x00\x12@\n\tAddWorker\x12\x19.bess.pb.AddWorkerRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12H\n\rDestroyWorker\x12\x1d.bess.pb.DestroyWorkerRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12;\n\x08ResetTcs\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12>\n\x07ListTcs\x12\x17.bess.pb.ListTcsRequest\x1a\x18.bess.pb.ListTcsResponse\"\x00\x12\x62\n\x1a\x43heckSchedulingConstraints\x12\x15.bess.pb.EmptyRequest\x1a+.bess.pb.CheckSchedulingConstraintsResponse\"\x00\x12\x38\n\x05\x41\x64\x64Tc\x12\x15.bess.pb.AddTcRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12J\n\x0eUpdateTcParams\x12\x1e.bess.pb.UpdateTcParamsRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12J\n\x0eUpdateTcParent\x12\x1e.bess.pb.UpdateTcParentRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12G\n\nGetTcStats\x12\x1a.bess.pb.GetTcStatsRequest\x1a\x1b.bess.pb.GetTcStatsResponse\"\x00\x12\x44\n\x0bListDrivers\x12\x15.bess.pb.EmptyRequest\x1a\x1c.bess.pb.ListDriversResponse\"\x00\x12P\n\rGetDriverInfo\x12\x1d.bess.pb.GetDriverInfoRequest\x1a\x1e.bess.pb.GetDriverInfoResponse\"\x00\x12=\n\nResetPorts\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12@\n\tListPorts\x12\x15.bess.pb.EmptyRequest\x1a\x1a.bess.pb.ListPortsResponse\"\x00\x12G\n\nCreatePort\x12\x1a.bess.pb.CreatePortRequest\x1a\x1b.bess.pb.CreatePortResponse\"\x00\x12\x44\n\x0b\x44\x65stroyPort\x12\x1b.bess.pb.DestroyPortRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x46\n\x0bSetPortConf\x12\x1b.bess.pb.SetPortConfRequest\x1a\x18.bess.pb.CommandResponse\"\x00\x12J\n\x0bGetPortConf\x12\x1b.bess.pb.GetPortConfRequest\x1a\x1c.bess.pb.GetPortConfResponse\"\x00\x12M\n\x0cGetPortStats\x12\x1c.bess.pb.GetPortStatsRequest\x1a\x1d.bess.pb.GetPortStatsResponse\"\x00\x12P\n\rGetLinkStatus\x12\x1d.bess.pb.GetLinkStatusRequest\x1a\x1e.bess.pb.GetLinkStatusResponse\"\x00\x12\x42\n\nListMclass\x12\x15.bess.pb.EmptyRequest\x1a\x1b.bess.pb.ListMclassResponse\"\x00\x12P\n\rGetMclassInfo\x12\x1d.bess.pb.GetMclassInfoRequest\x1a\x1e.bess.pb.GetMclassInfoResponse\"\x00\x12?\n\x0cResetModules\x12\x15.bess.pb.EmptyRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12\x44\n\x0bListModules\x12\x15.bess.pb.EmptyRequest\x1a\x1c.bess.pb.ListModulesResponse\"\x00\x12M\n\x0c\x43reateModule\x12\x1c.bess.pb.CreateModuleRequest\x1a\x1d.bess.pb.CreateModuleResponse\"\x00\x12H\n\rDestroyModule\x12\x1d.bess.pb.DestroyModuleRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12P\n\rGetModuleInfo\x12\x1d.bess.pb.GetModuleInfoRequest\x1a\x1e.bess.pb.GetModuleInfoResponse\"\x00\x12J\n\x0e\x43onnectModules\x12\x1e.bess.pb.ConnectModulesRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12P\n\x11\x44isconnectModules\x12!.bess.pb.DisconnectModulesRequest\x1a\x16.bess.pb.EmptyResponse\"\x00\x12J\n\x0b\x44umpMempool\x12\x1b.bess.pb.DumpMempoolRequest\x1a\x1c.bess.pb.DumpMempoolResponse\"\x00\x12\x44\n\rModuleCommand\x12\x17.bess.pb.CommandRequest\x1a\x18.bess.pb.CommandResponse\"\x00\x12P\n\x11ListGateHookClass\x12\x15.bess.pb.EmptyRequest\x1a\".bess.pb.ListGateHookClassResponse\"\x00\x12\x65\n\x14GetGateHookClassInfo\x12$.bess.pb.GetGateHookClassInfoRequest\x1a%.bess.pb.GetGateHookClassInfoResponse\"\x00\x12\\\n\x11\x43onfigureGateHook\x12!.bess.pb.ConfigureGateHookRequest\x1a\".bess.pb.ConfigureGateHookResponse\"\x00\x12H\n\rListGateHooks\x12\x15.bess.pb.EmptyRequest\x1a\x1e.bess.pb.ListGateHooksResponse\"\x00\x12N\n\x0fGateHookCommand\x12\x1f.bess.pb.GateHookCommandRequest\x1a\x18.bess.pb.CommandResponse\"\x00\x12V\n\x13\x43onfigureResumeHook\x12#.bess.pb.ConfigureResumeHookRequest\x1a\x18.bess.pb.CommandResponse\"\x00\x62\x06proto3'
  ,
  dependencies=[bess__msg__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_BESSCONTROL = _descriptor.ServiceDescriptor(
  name='BESSControl',
  full_name='bess.pb.BESSControl',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=43,
  serialized_end=3583,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetVersion',
    full_name='bess.pb.BESSControl.GetVersion',
    index=0,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._VERSIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResetAll',
    full_name='bess.pb.BESSControl.ResetAll',
    index=1,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='KillBess',
    full_name='bess.pb.BESSControl.KillBess',
    index=2,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ImportPlugin',
    full_name='bess.pb.BESSControl.ImportPlugin',
    index=3,
    containing_service=None,
    input_type=bess__msg__pb2._IMPORTPLUGINREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UnloadPlugin',
    full_name='bess.pb.BESSControl.UnloadPlugin',
    index=4,
    containing_service=None,
    input_type=bess__msg__pb2._UNLOADPLUGINREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListPlugins',
    full_name='bess.pb.BESSControl.ListPlugins',
    index=5,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTPLUGINSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PauseAll',
    full_name='bess.pb.BESSControl.PauseAll',
    index=6,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PauseWorker',
    full_name='bess.pb.BESSControl.PauseWorker',
    index=7,
    containing_service=None,
    input_type=bess__msg__pb2._PAUSEWORKERREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResumeWorker',
    full_name='bess.pb.BESSControl.ResumeWorker',
    index=8,
    containing_service=None,
    input_type=bess__msg__pb2._RESUMEWORKERREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResumeAll',
    full_name='bess.pb.BESSControl.ResumeAll',
    index=9,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResetWorkers',
    full_name='bess.pb.BESSControl.ResetWorkers',
    index=10,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListWorkers',
    full_name='bess.pb.BESSControl.ListWorkers',
    index=11,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTWORKERSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AddWorker',
    full_name='bess.pb.BESSControl.AddWorker',
    index=12,
    containing_service=None,
    input_type=bess__msg__pb2._ADDWORKERREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DestroyWorker',
    full_name='bess.pb.BESSControl.DestroyWorker',
    index=13,
    containing_service=None,
    input_type=bess__msg__pb2._DESTROYWORKERREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResetTcs',
    full_name='bess.pb.BESSControl.ResetTcs',
    index=14,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListTcs',
    full_name='bess.pb.BESSControl.ListTcs',
    index=15,
    containing_service=None,
    input_type=bess__msg__pb2._LISTTCSREQUEST,
    output_type=bess__msg__pb2._LISTTCSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CheckSchedulingConstraints',
    full_name='bess.pb.BESSControl.CheckSchedulingConstraints',
    index=16,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._CHECKSCHEDULINGCONSTRAINTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='AddTc',
    full_name='bess.pb.BESSControl.AddTc',
    index=17,
    containing_service=None,
    input_type=bess__msg__pb2._ADDTCREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateTcParams',
    full_name='bess.pb.BESSControl.UpdateTcParams',
    index=18,
    containing_service=None,
    input_type=bess__msg__pb2._UPDATETCPARAMSREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateTcParent',
    full_name='bess.pb.BESSControl.UpdateTcParent',
    index=19,
    containing_service=None,
    input_type=bess__msg__pb2._UPDATETCPARENTREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetTcStats',
    full_name='bess.pb.BESSControl.GetTcStats',
    index=20,
    containing_service=None,
    input_type=bess__msg__pb2._GETTCSTATSREQUEST,
    output_type=bess__msg__pb2._GETTCSTATSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListDrivers',
    full_name='bess.pb.BESSControl.ListDrivers',
    index=21,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTDRIVERSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetDriverInfo',
    full_name='bess.pb.BESSControl.GetDriverInfo',
    index=22,
    containing_service=None,
    input_type=bess__msg__pb2._GETDRIVERINFOREQUEST,
    output_type=bess__msg__pb2._GETDRIVERINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResetPorts',
    full_name='bess.pb.BESSControl.ResetPorts',
    index=23,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListPorts',
    full_name='bess.pb.BESSControl.ListPorts',
    index=24,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTPORTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreatePort',
    full_name='bess.pb.BESSControl.CreatePort',
    index=25,
    containing_service=None,
    input_type=bess__msg__pb2._CREATEPORTREQUEST,
    output_type=bess__msg__pb2._CREATEPORTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DestroyPort',
    full_name='bess.pb.BESSControl.DestroyPort',
    index=26,
    containing_service=None,
    input_type=bess__msg__pb2._DESTROYPORTREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SetPortConf',
    full_name='bess.pb.BESSControl.SetPortConf',
    index=27,
    containing_service=None,
    input_type=bess__msg__pb2._SETPORTCONFREQUEST,
    output_type=bess__msg__pb2._COMMANDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetPortConf',
    full_name='bess.pb.BESSControl.GetPortConf',
    index=28,
    containing_service=None,
    input_type=bess__msg__pb2._GETPORTCONFREQUEST,
    output_type=bess__msg__pb2._GETPORTCONFRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetPortStats',
    full_name='bess.pb.BESSControl.GetPortStats',
    index=29,
    containing_service=None,
    input_type=bess__msg__pb2._GETPORTSTATSREQUEST,
    output_type=bess__msg__pb2._GETPORTSTATSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetLinkStatus',
    full_name='bess.pb.BESSControl.GetLinkStatus',
    index=30,
    containing_service=None,
    input_type=bess__msg__pb2._GETLINKSTATUSREQUEST,
    output_type=bess__msg__pb2._GETLINKSTATUSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListMclass',
    full_name='bess.pb.BESSControl.ListMclass',
    index=31,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTMCLASSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetMclassInfo',
    full_name='bess.pb.BESSControl.GetMclassInfo',
    index=32,
    containing_service=None,
    input_type=bess__msg__pb2._GETMCLASSINFOREQUEST,
    output_type=bess__msg__pb2._GETMCLASSINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ResetModules',
    full_name='bess.pb.BESSControl.ResetModules',
    index=33,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListModules',
    full_name='bess.pb.BESSControl.ListModules',
    index=34,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTMODULESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateModule',
    full_name='bess.pb.BESSControl.CreateModule',
    index=35,
    containing_service=None,
    input_type=bess__msg__pb2._CREATEMODULEREQUEST,
    output_type=bess__msg__pb2._CREATEMODULERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DestroyModule',
    full_name='bess.pb.BESSControl.DestroyModule',
    index=36,
    containing_service=None,
    input_type=bess__msg__pb2._DESTROYMODULEREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetModuleInfo',
    full_name='bess.pb.BESSControl.GetModuleInfo',
    index=37,
    containing_service=None,
    input_type=bess__msg__pb2._GETMODULEINFOREQUEST,
    output_type=bess__msg__pb2._GETMODULEINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ConnectModules',
    full_name='bess.pb.BESSControl.ConnectModules',
    index=38,
    containing_service=None,
    input_type=bess__msg__pb2._CONNECTMODULESREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DisconnectModules',
    full_name='bess.pb.BESSControl.DisconnectModules',
    index=39,
    containing_service=None,
    input_type=bess__msg__pb2._DISCONNECTMODULESREQUEST,
    output_type=bess__msg__pb2._EMPTYRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DumpMempool',
    full_name='bess.pb.BESSControl.DumpMempool',
    index=40,
    containing_service=None,
    input_type=bess__msg__pb2._DUMPMEMPOOLREQUEST,
    output_type=bess__msg__pb2._DUMPMEMPOOLRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ModuleCommand',
    full_name='bess.pb.BESSControl.ModuleCommand',
    index=41,
    containing_service=None,
    input_type=bess__msg__pb2._COMMANDREQUEST,
    output_type=bess__msg__pb2._COMMANDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListGateHookClass',
    full_name='bess.pb.BESSControl.ListGateHookClass',
    index=42,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTGATEHOOKCLASSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GetGateHookClassInfo',
    full_name='bess.pb.BESSControl.GetGateHookClassInfo',
    index=43,
    containing_service=None,
    input_type=bess__msg__pb2._GETGATEHOOKCLASSINFOREQUEST,
    output_type=bess__msg__pb2._GETGATEHOOKCLASSINFORESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ConfigureGateHook',
    full_name='bess.pb.BESSControl.ConfigureGateHook',
    index=44,
    containing_service=None,
    input_type=bess__msg__pb2._CONFIGUREGATEHOOKREQUEST,
    output_type=bess__msg__pb2._CONFIGUREGATEHOOKRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListGateHooks',
    full_name='bess.pb.BESSControl.ListGateHooks',
    index=45,
    containing_service=None,
    input_type=bess__msg__pb2._EMPTYREQUEST,
    output_type=bess__msg__pb2._LISTGATEHOOKSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='GateHookCommand',
    full_name='bess.pb.BESSControl.GateHookCommand',
    index=46,
    containing_service=None,
    input_type=bess__msg__pb2._GATEHOOKCOMMANDREQUEST,
    output_type=bess__msg__pb2._COMMANDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ConfigureResumeHook',
    full_name='bess.pb.BESSControl.ConfigureResumeHook',
    index=47,
    containing_service=None,
    input_type=bess__msg__pb2._CONFIGURERESUMEHOOKREQUEST,
    output_type=bess__msg__pb2._COMMANDRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_BESSCONTROL)

DESCRIPTOR.services_by_name['BESSControl'] = _BESSCONTROL

# @@protoc_insertion_point(module_scope)
